
/****************************************************
 *   Proj.Name: C - Interface for                   *
 *   PI C-842 DC Motor Controller                   *
 *   Version: 3.10                                  *
 *   Date:    19.04.99                              *
 ****************************************************
 *                                                  *
 *                                                  *
 *  History:                           Date:        *
 * -----------------------------------------------  *
 *  First Implementation               28.11.1995   *
 *  added Function: Init_LS()          13.01.1996   *
 *  BugFix SV und TY                   05.02.1996   *
 *  BugFix TL                          08.02.1996   *
 *  V 2.3  modified for 2 and 4 axis   29.03.1996   *
 *         file date                   15.04.96     *
 *  V 3.1  corrections V31             19.04.99     * 
 ****************************************************/

#ifndef unix
#include <conio.h>
#include <dos.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "linux_qflc310x.h"

#ifdef unix
#include <ctype.h>
#include <unistd.h>
#define outportb(a,b)	outb(b,a)
#define inportb	inb
#define inport	inw
#define getch		getchar
//implement the delay function
void delay(int);

void delay(int d){
//implementation using usleep
usleep(1000l*(long)d);      
/*
// implementation using 'nanosleep'
        timespec t,r;
        r.tv_sec=0;
        r.tv_nsec=0;
        t.tv_sec=0;
        t.tv_nsec=(unsigned long)delay*1000000ul;
        nanosleep(&t,&r); 
*/}


//change the dos style outportb to its linux equivalent
//and include the correct libraries

//Update, 23.5.01, Steve:
//ioperm is now in <sys/io.h> rather than <asm/io.h>
//since we update the glibc libraries.

#include <sys/io.h> /* for glibc */       
#include <unistd.h> /* for libc5 */
void strupr(char *);
#define MAXSTRUPR	256
void strupr(char *s){
	INT i=0;
	do{
		s[i]=toupper(s[i]);
		}while((s[i++]!='\0')&&(i<MAXSTRUPR));
	}
#endif
#ifndef unix
#define INT int
#endif

long 	tpos[5]     ={0,0,0,0,0};
long 	poserror[5] ={0,0,0,0,0};
long	PosEx[5];
char 	rep[255];
INT  	status[5]   ={0,0,0,0,0};
BYTE acta = 0xFF;
BYTE MaxAxis = 4;
REG Reg={0x210, 0x211, 0x212, 0x213};


char *ver="(C) by PI: QFLC_X300: Version: 3.1";

void Set_BaseAddress(unsigned long address)
{	
	INT ret=0;
	Reg.Data = address;
	Reg.Cmd  = address + 1;
	Reg.Func = address + 2;
	Reg.IO   = address + 3;
//	printf("In Set_BaseAddress %x\n",address);
#ifdef unix
//	Add permission for the process to write to the I/O port
//	Note that this requires that the process runs as "root"
// 	and as such is a security risk (especially if the user
//	code is "weak" as is the usual case)
	ret=ioperm(Reg.Data,4,3);
//	printf("Linux: Setting permissions\n");
	if(ret!=0){
		printf("Error: in Set_BaseAddress, ioperm was not able to set appropriate\n");
		printf("       permissions for A2D board address %x, perhaps you are not \"root\"\n",Reg.Data);
		printf("       This means a segmentation fault is likely to occur\n");
		}
	else{
//		printf("Permission set ok\n");
		}
#endif
	return;
}

BYTE Axis_Installed()
{
	long ver;
	BYTE num;

	ver = getQMC(0x6C);
	ver &= 0x3800;
	ver >>= 11;
	num = (unsigned) (ver +1);
	if(num != 4) setQMC(0x38,4);   // 400s Sample time
	MaxAxis = (BYTE)num;
	return(num);
}

void Init_LS(BYTE ax1, BYTE ax2, BYTE ax3, BYTE ax4)
{
	unsigned sense = 0xFF;

	if(ax1) sense &= ~0x03;
	if(ax2) sense &= ~0x0C;
	if(ax3) sense &= ~0x30;
	if(ax4) sense &= ~0xC0;
	setQMC(0x66, sense);
	return;
}

INT Board_Installed()
{
	long ret;

	ret = getQMC(0x6C);
	if((ret & 0xFFE0) == 0x5900) return(TRUE);			//check Version
	return(FALSE);
}

BYTE moving(BYTE axis)
{
        if(axis != acta) Select(axis);           /*V31*/ 
//	Select(axis);
//	if(getQMC(0x31) & 0x01) return(FALSE);   /*V31*/
	if(getQMC(0x31) & 0x400) return(TRUE);
	else return (FALSE);
}

void translate(char *comstr, char *report)
{
	INT i, axis, sign;
	long para;
	char command[20], chelp[20];
	char ParaStr[40];
	char rep[80];
	BYTE start;
	TCMD cmd;

	sprintf(rep,"No Report");
	command[0] = '\0';
	ParaStr[0] = '\0';
	sign = FALSE;

	while(comstr[0] == ' '){                    	// delete left blanks
		i = 1;
		do{
			comstr[i-1] = comstr[i++];
		}while(comstr[i] != '\0' || i < 20);
	}
	axis = 0;
	switch(comstr[0]){                         // get axis ident.
		case '9':   axis++;
		case '8':   axis++;
		case '7':   axis++;
		case '6':   axis++;
		case '5':   axis++;
		case '4':	axis++;
		case '3':	axis++;
		case '2':	axis++;
		case '1':	axis++;
		case '0':   if(axis > MaxAxis){
						sprintf(report,"ERROR in [translate]: wrong axis specifier\0");
						return;
					}
					start = 1;
					break;
		default :   axis = 0;
					start = 0;
					break;
	}
	for(i=start; i < strlen(comstr); i++){         // check for valid characters
		if(comstr[i] >= 'A' && comstr[i] <= 'z'){
			strcpy(chelp, command);
			sprintf(command,"%s%c\0",chelp,comstr[i]); // extract command string
		}
		if(comstr[i] >= '0' && comstr[i] <= '9'){
			strcpy(chelp, ParaStr);
			sprintf(ParaStr,"%s%c",chelp,comstr[i]);// extract parameter string
		}
		if(comstr[i] == '-') sign = TRUE;			// extract sign
	}
	strupr(command);                                // all chars to upper
	para = atol(ParaStr);							// convert para-string to long
	if(sign) para = -para;                  		// get correct sign
	if(strlen(command) == 2){                       // check for 2char command
			 if(!strcmp(command,"AB\0")) cmd = AB;
		else if(!strcmp(command,"MA\0")) cmd = MA;
		else if(!strcmp(command,"MR\0")) cmd = MR;
		else if(!strcmp(command,"DH\0")) cmd = DH;
		else if(!strcmp(command,"GH\0")) cmd = GH;
		else if(!strcmp(command,"TT\0")) cmd = TT;
		else if(!strcmp(command,"TP\0")) cmd = TP;
		else if(!strcmp(command,"TS\0")) cmd = TS;
		else if(!strcmp(command,"TL\0")) cmd = TL;
		else if(!strcmp(command,"MN\0")) cmd = MN;
//10
		else if(!strcmp(command,"MF\0")) cmd = MF;
		else if(!strcmp(command,"CN\0")) cmd = CN;
		else if(!strcmp(command,"CF\0")) cmd = CF;
		else if(!strcmp(command,"SA\0")) cmd = SA;
		else if(!strcmp(command,"SV\0")) cmd = SV;
		else if(!strcmp(command,"SL\0")) cmd = SL;
		else if(!strcmp(command,"IV\0")) cmd = IV;
		else if(!strcmp(command,"ST\0")) cmd = ST;
		else if(!strcmp(command,"RT\0")) cmd = RT;
		else if(!strcmp(command,"WS\0")) cmd = WS;
//20
		else if(!strcmp(command,"WA\0")) cmd = WA;
		else if(!strcmp(command,"TG\0")) cmd = TG;
		else if(!strcmp(command,"TC\0")) cmd = TC;
		else if(!strcmp(command,"TE\0")) cmd = TE;
		else if(!strcmp(command,"TF\0")) cmd = TF;
//		else if(!strcmp(command,"TI\0")) cmd = TI;
		else if(!strcmp(command,"TX\0")) cmd = TX;
		else if(!strcmp(command,"TY\0")) cmd = TY;
		else if(!strcmp(command,"TV\0")) cmd = TV;
		else if(!strcmp(command,"VE\0")) cmd = VE;
//30
		else if(!strcmp(command,"DP\0")) cmd = DP;
		else if(!strcmp(command,"DI\0")) cmd = DI;
		else if(!strcmp(command,"DD\0")) cmd = DD;
		else if(!strcmp(command,"DL\0")) cmd = DL;
		else if(!strcmp(command,"GP\0")) cmd = GP;
		else if(!strcmp(command,"GI\0")) cmd = GI;
		else if(!strcmp(command,"GD\0")) cmd = GD;
		else if(!strcmp(command,"GL\0")) cmd = GL;
		else if(!strcmp(command,"TZ\0")) cmd = TZ;
		else if(!strcmp(command,"AN\0")) cmd = AN;
//40
		else if(!strcmp(command,"AF\0")) cmd = AF;
		else if(!strcmp(command,"DF\0")) cmd = DF;
		else if(!strcmp(command,"DV\0")) cmd = DV;
		else if(!strcmp(command,"GB\0")) cmd = GB;
		else if(!strcmp(command,"GF\0")) cmd = GF;
		else if(!strcmp(command,"GJ\0")) cmd = GJ;
		else if(!strcmp(command,"GT\0")) cmd = GT;
		else if(!strcmp(command,"GV\0")) cmd = GV;
		else if(!strcmp(command,"LF\0")) cmd = LF;
		else if(!strcmp(command,"LN\0")) cmd = LN;
//50
		else if(!strcmp(command,"SB\0")) cmd = SB;
		else if(!strcmp(command,"SJ\0")) cmd = SJ;
		else if(!strcmp(command,"WC\0")) cmd = WC;
		else if(!strcmp(command,"WT\0")) cmd = WT;
		else{
			sprintf(report,"ERROR in [translate]: Command not found: %s",command);
			return;
		}
	} // strlen == 2
	else{                                           // 3 char commands
                if(!strcmp(command,"GLS\0")) cmd = GLS;
		else if(!strcmp(command,"GIP\0")) cmd = GIP;
		else if(!strcmp(command,"GMO\0")) cmd = GMO;
		else if(!strcmp(command,"SRA\0")) cmd = SRA;
		else if(!strcmp(command,"GRA\0")) cmd = GRA;
		else if(!strcmp(command,"CLR\0")) cmd = CLR;
		else if(!strcmp(command,"GPP\0")) cmd = GPP;
		else if(!strcmp(command,"GPV\0")) cmd = GPV;
		else if(!strcmp(command,"GIR\0")) cmd = GIR;
		else if(!strcmp(command,"SAN\0")) cmd = SAN;
//10
		else if(!strcmp(command,"ASN\0")) cmd = ASN;
		else if(!strcmp(command,"ASF\0")) cmd = ASF;
		else if(!strcmp(command,"SMO\0")) cmd = SMO;
		else if(!strcmp(command,"SFP\0")) cmd = SFP;
		else if(!strcmp(command,"SFV\0")) cmd = SFV;
		else if(!strcmp(command,"GPE\0")) cmd = GPE;
		else if(!strcmp(command,"GSI\0")) cmd = GSI;
		else if(!strcmp(command,"GIM\0")) cmd = GIM;
		else if(!strcmp(command,"SCI\0")) cmd = SCI;
		else if(!strcmp(command,"SPS\0")) cmd = SPS;
//20
		else if(!strcmp(command,"SPT\0")) cmd = SPT;
		else if(!strcmp(command,"SMA\0")) cmd = SMA;
		else if(!strcmp(command,"GMA\0")) cmd = GMA;
		else if(!strcmp(command,"CLP\0")) cmd = CLP;
		else if(!strcmp(command,"SPE\0")) cmd = SPE;
		else if(!strcmp(command,"SCH\0")) cmd = SCH;
		else if(!strcmp(command,"SLS\0")) cmd = SLS;
		else if(!strcmp(command,"GTI\0")) cmd = GTI;
		else if(!strcmp(command,"SBT\0")) cmd = SBT;
		else if(!strcmp(command,"SBP\0")) cmd = SBP;
//30
		else if(!strcmp(command,"SBN\0")) cmd = SBN;
		else if(!strcmp(command,"SBA\0")) cmd = SBA;
		else if(!strcmp(command,"SBC\0")) cmd = SBC;
		else if(!strcmp(command,"SBF\0")) cmd = SBF;
		else if(!strcmp(command,"SIM\0")) cmd = SIM;
		else if(!strcmp(command,"SPV\0")) cmd = SPV;
		else if(!strcmp(command,"SPG\0")) cmd = SPG;
		else if(!strcmp(command,"SOP\0")) cmd = SOP;
		else if(!strcmp(command,"SOT\0")) cmd = SOT;
		else if(!strcmp(command,"SOH\0")) cmd = SOH;
//40
		else if(!strcmp(command,"SST\0")) cmd = SST;
		else if(!strcmp(command,"GOM\0")) cmd = GOM;
		else if(!strcmp(command,"RTI\0")) cmd = RTI;
		else if(!strcmp(command,"SAF\0")) cmd = SAF;
		else if(!strcmp(command,"SYN\0")) cmd = SYN;
		else if(!strcmp(command,"FEP\0")) cmd = FEP;
		else if(!strcmp(command,"FEN\0")) cmd = FEN;
		else{
			sprintf(report,"ERROR in [translate]: Command not found: %s",command);
			return;
		}
	}
	execute(axis, cmd, para, rep);                 // execute command
	strcpy(report, rep);                           // copy report

	return;
}

void ClearAxisStatus(BYTE axis)
{
	Select(axis);
	setQMC(0x33,0);
	return;
}

void InitAxis(BYTE axis)         // set axis default values
{
	INT i;

	if(axis > 0 ) i = axis;     // if axis 1..4
	else i = 1;
	do{
		Select(i);
		setQMC(0x33,  0);       // Clear Status
		setQMC(0x25, 60);       // Set DP
		setQMC(0x11, 120000L);  // Set Velocity
		setQMC(0x12, 5000);     // Set Acceleration
		setQMC(0x44,  0);       // Auto Stop off
		setQMC(0x1A,  0);       // Update Filter
		setQMC(0x33,  0);       // Clear Status
		if(!axis) i++;          // if axis = 0
		else i = 5;
	}while(i < 5);              // repeat for all axis
	return;
}

/********************************************************
 *														*
 * Function: InitBoard					*
 ********************************************************
 *							*
 * Returns Status Byte	or -1 on error			*
 *                                                  	*
 * Mode = Board Function Register			*
 * 		*Bit 0 (0x01) : 12V Motor Supply		*
 *		 Bit 1 (0x02) : RS422 (DEM) active for #1, #2   *
 *		 Bit 2 (0x04) : RS422 (DEM) active for #3, #4	*
 *		*Bit 3 (0x08) : Pull Up Limit switch		*
 *		 Bit 4 (0x10) : Enable IRQ 10			*
 *		 Bit 5 (0x20) : Enable IRQ 11			*
 *		 Bit 6 (0x40) : Enable IRQ 12			*
 *		 Bit 7 (0x80) : Enable IRQ 15			*
 *		*=default setting				*
 ********************************************************/
INT InitBoard(BYTE mode)
{	int	i=0;
	if(QMCReady()) outportb(Reg.Cmd, 0x39);
	else return(-1);

	setQMC(0x3B,0);					// Set Output DAC 1 6
	setQMC(0x66,0x00FF);			// Set Limit Sense 0x00FF, LS open
									// 				   0x0000, DEM circuit
	//printf("debug %d\n",i++);
	delay(1);
	//printf("debug %d\n",i++);
	outportb(Reg.Func, mode);
	//printf("debug %d\n",i++);
	return(inportb(Reg.Cmd));
}

//----------------------------------------------------------------
// execute: a comand in TCMD-format will be executed                  |
// axis:	specifies the axis for which the command is valid     |
//			if axis = 0 the command is valid for all axis |
// para:	contains the parameter value such as velocity...      |
// report:	contains the report after the function is ready       |
//-----------------------------------------------------------------
INT execute(BYTE axis, TCMD command, long para, char *report)
{
	INT i, sign, enough, key;
	INT hiint, loint, wertint;
	static unsigned outbyte, inbyte;
	unsigned shbyte;
	unsigned LoWord;
	unsigned vword;
	long L_HiWord;
	long lret;
	long currpos;
	long para2;
	long vlong;
	long templong;
	long paratrans;
	long parafrac;
	long getval;

	float tempfloat;
	char strbin[10];
	char shelp[80];
	ldiv_t paraint;

	if(axis > MaxAxis){
		sprintf(report,"ERROR in [execute]: wrong axis specifier\0");
		return(0);
	}
	else sprintf(report," \0");

	if(command.sect == CMD_NOPARA){
		if(axis == 0) i = 1;         // if axis=0 execute cmd to all axis
		else i = axis;
		do{
			if(i != acta) Select(i);
			setQMC(command.code, NOPARAM);
			strcpy(shelp,report);       // copy previous report
										// concat current report to prev.
			sprintf(report, "%s %d %s", shelp, i, command.lab);
			if(command.code == 0x3F){
				setQMC(0x10, NOPARAM);
				setQMC(0x1A, NOPARAM);
				PosEx[i] = 0;
			}
			if(axis == 0) i++;          // for single axis command
			else i = MaxAxis + 1;                 // set break condition
		}while(i < MaxAxis);
	} // end of CMD_NOPARA
	else if(command.sect == RD_NOFRAC){
		if(axis > 0 && axis <= MaxAxis){
			if(axis != acta) Select(axis);
			lret = getQMC(command.code);
			sprintf(report, "%d %s %ld", axis, command.lab, lret);
		}
		if(!axis){
			for(i=1; i <= MaxAxis; i++){
				Select(i);
				lret = getQMC(command.code);
				strcpy(shelp, report);
				sprintf(report, "%s %d %s %ld", shelp, i, command.lab, lret);
			}
		}
	} // end of RD_NOFRAC

	else if(command.sect == WR_NOFRAC){
		if(axis > 0 && axis <= MaxAxis){
			if(axis != acta) Select(axis);
			setQMC(command.code, para);
			sprintf(report, "%d %s", axis, command.lab);
			setQMC(0x1A, NOPARAM);   /*V31*/
		}
		if(!axis){                         // just another way to
			for(i=1; i <= MaxAxis; i++){          // execute multi axis commands
				Select(i);
				setQMC(command.code, para);
				strcpy(shelp, report);
				sprintf(report, "%s %d %s", shelp, i, command.lab);
			}
			setQMC(0x5B, 0x0F);
		}
	} // end of WR_NOFRAC
	else{
	  switch(command.code){
		case 0xE5:
		case 0xEF:	Read_PES(tpos, poserror, status);
					if(axis > 0 && axis <= MaxAxis){
						if(axis != acta) Select(axis);
						setQMC(MA.code, tpos[axis]);
						sprintf(report, "%d %s", axis, command.lab);
						setQMC(0x33, NOPARAM);
						setQMC(0x1A, NOPARAM);
					}
					if(!axis){
						for(i=1; i <= MaxAxis; i++){
							Select(i);
							setQMC(MA.code, tpos[i]);
							strcpy(shelp, report);
							sprintf(report, "%s %d %s", shelp, i, command.lab);
							setQMC(0x33, NOPARAM);
						}
						setQMC(0x5B, 0x0F);
					}
					break;
		case 0xE1:
		case 0x10:
					if(axis > 0 && axis <= MaxAxis){
						if(axis != acta) Select(axis);
						if(command.code == 0xE1){
							currpos = getQMC(TT.code);
							para   += currpos;
						}
						setQMC(MA.code, para);
						sprintf(report, "%d %s", axis, command.lab);
						setQMC(0x33, NOPARAM);
						setQMC(0x1A, NOPARAM);
					}
					if(!axis){
						for(i=1; i <= MaxAxis; i++){
							Select(i);
							if(command.code == 0xE1){
								currpos = getQMC(TT.code);
								para2   = para + currpos;
							}
							else para2 = para;
							setQMC(MA.code, para2);
							strcpy(shelp, report);
							sprintf(report, "%s %d %s", shelp, i, command.lab);
							setQMC(0x33, NOPARAM);
						}
						setQMC(0x5B, 0x0F);
					}
					break;
		case 0xE2:  if(axis > 0 && axis <= MaxAxis){
						if(axis != acta) Select(axis);
						setQMC(MA.code, 0);
						sprintf(report, "%d %s", axis, command.lab);
						setQMC(0x33, NOPARAM);
						setQMC(0x1A, NOPARAM);
					}
					if(!axis){
						for(i=1; i <= MaxAxis; i++){
							Select(i);
							setQMC(MA.code, 0);
							PosEx[i] = 0;
							strcpy(shelp, report);
							sprintf(report, "%s %d %s", shelp, i, command.lab);
							setQMC(0x33, NOPARAM);
						}
						setQMC(0x5B, 0x0F);
					}
					break;
		case 0xE4:  					// Read actual running speed
					if(axis > 0 ) i = axis;
					else i = 1;
					do{
						Select(i);
						vlong = getQMC(TP.code);
						delay(20);      // v=s/t t = 20ms
						vlong = (getQMC(TP.code) - vlong)*50;
						strcpy(shelp, report);
						sprintf(report, "%s %d %s %ld", shelp, i, command.lab, vlong);
						if(!axis) i++;
						else i = MaxAxis +1;
					}while(i <= MaxAxis);
					break;
		case 0x1E:  // GPV: Format S1616
		case 0x4C:  // TL: Get Acceleration
					if(axis > 0 ) i = axis;
					else i = 1;
					do{
						Select(i);
						vlong    = getQMC(command.code);
						LoWord   = (unsigned) vlong;
						L_HiWord = vlong >> 16;
						tempfloat= (float) LoWord;
						vword    = (unsigned) floor((tempfloat/26.2144)+0.5);
						templong = (L_HiWord * 2500) + vword;
						strcpy(shelp, report);
						sprintf(report, "%s %d %s %ld", shelp, i, command.lab, templong);
						if(!axis) i++;
						else i = MaxAxis +1;
					}while(i <= MaxAxis);
					break;
		case 0x4B: 				// TY,TL: Format U1616
								// Sample time: 400ms == 2500 Samples/s
								// Param. calc. for 1/s
					if(axis > 0 ) i = axis;
					else i = 1;
					do{
						Select(i);
						vlong    = getQMC(command.code);
						LoWord   = (unsigned) vlong;
						L_HiWord = vlong >> 16;
						tempfloat= (float)(LoWord);
						vword    = (unsigned)floor((tempfloat/26.2144)+0.5); // 65536/2500
						templong = (L_HiWord * 2500) + vword;
						strcpy(shelp, report);
						sprintf(report, "%s %d %s %ld ", shelp, i, command.lab, templong);
						if(!axis) i++;
						else i = MaxAxis +1;
					}while(i <= MaxAxis);
					break;
		case 0x11:  // SV: Set Velocity
		case 0x12:	// SA: Set Acceleration
		case 0xEE:	// IV: Increment Velocity
					if(para < 0) sign = -1;
					else sign = 1;
					para = labs(para);
					paraint = ldiv(para, 2500L);	//paraint.quot = para/2500L
													// paraint.rem = para%2500L
					tempfloat= (float)paraint.rem;
					parafrac = (unsigned)floor((tempfloat / 2500.0 * 65536.0)+0.5);
				  //	tempfloat= (float)floor((tempfloat / 2500.0 * 65536.0)+0.5);
				  //	parafrac = (int) tempfloat;
								// round = floor(value+0.5);
					paratrans= paraint.quot * 65536L + parafrac;
					if(command.code == 0x12) paratrans *= sign;
					if(axis > 0 ) i = axis;
					else i = 1;
					report[0] = '\0';
					do{
						Select(i);
						if(command.code == 0xEE)
							setQMC(0x11, getQMC(TY.code) + paratrans);
						else
							setQMC(command.code, paratrans);
							strcpy(shelp,report);
						sprintf(report, "%s %d %s ", shelp, i, command.lab);
						if(!axis){
							setQMC(0x5B, 0x0F);
							i++;
						}
						else{
							setQMC(0x1A, NOPARAM);
							i = MaxAxis +1;
						}
					}while(i <= MaxAxis);
					break;
		case 0x14:	// SRA: Set Ratio, Format S1616
					// Der bertragene Parameter entspr. 1/1000 des Int-Wertes
					paraint.quot = (long) para/1000;
					tempfloat  = ((float)(para)) / 1000.0 - paraint.quot;
					parafrac   = (unsigned) floor(tempfloat * 65536.0 - 0.5);
					paratrans  = paraint.quot * 65536L + parafrac;
					report[0]  = '\0';
					if(axis > 0 ) i = axis;
					else i = 1;
					do{
						Select(i);
						setQMC(command.code, paratrans);
						strcpy(shelp, report);
						sprintf(report, "%s %d %s", shelp, i, command.lab);
						if(!axis) i++;
						else i = MaxAxis/2 +1;
					}while(i <= (MaxAxis/2));
					setQMC(0x1A, NOPARAM);
					break;
		case 0x59:	// GRA: S1616
					report[0] = '\0';
					if(axis > 0 ) i = axis;
					else i = 1;
					do{
						Select(i);
						getval = getQMC(command.code);
						hiint  = (unsigned)(getval/65536L);
						loint  = (unsigned)(floor((float)((getval-(hiint*65536L))/65536L)*1000));
								// round = floor(value+0.5);
						wertint= hiint*1000 + loint;
						strcpy(shelp, report);
						sprintf(report, "%s %d %s %d", shelp, i, command.lab, wertint);
						if(!axis) i++;
						else i = MaxAxis/2 +1;
					}while(i <= MaxAxis/2);
					break;
		case 0xEA:	// TZ: Tell Channel Input in Dezimal Notation
					sprintf(report,"INPUT: %d ", inportb(Reg.IO));
					break;
		case 0xEB: // TC: Tell Channel Input in Binary Notation
					Byte2BinStr(inportb(Reg.IO), strbin);
					sprintf(report,"INPUT: %s ", strbin);
					break;
		case 0xEC: // CN: Channel ON
					shbyte = 0x01;
					if(para >= 0 && para <= 8){
						if(para == 0) outbyte = 0xFF;
						else outbyte |= (shbyte << (para-1));
						outportb(Reg.IO, outbyte);
						Byte2BinStr(outbyte, strbin);
						sprintf(report,"OUTPUT: %s ", strbin);
					}
					else sprintf(report,"ERROR: Wrong Channel specified");
					break;
		case 0xED:  // CF: Channel OFF
					shbyte = 0x01;
					if(para >= 0 && para <= 8){
						if(para == 0) outbyte = 0xFF;
						else outbyte &= ~(shbyte << (para-1));
						outportb(Reg.IO, outbyte);
						Byte2BinStr(outbyte, strbin);
						sprintf(report,"OUTPUT: %s ", strbin);
					}
					else sprintf(report,"ERROR: Wrong Channel specified");
					break;
//----------------------------------------------------------------------------------
		case 0xE3:	// WS: Wait for Stop
		case 0xF1:	// WT: Wait for Stop then trigger on channel para
			printf("WS and WT are not implemented under linux \n");
	
			/*		if(axis <= MaxAxis){
						if(axis > 0) i = axis;
						else i = 1;
						do{
							Select(i);
							enough = FALSE;
							do{
								delay(45);
								_AH = 0x02;
								geninterrupt(0x16);
								if((_AL&0x06) == 0x06) enough = TRUE;
							}while(getQMC(GPP.code) != getQMC(TT.code) && !enough);
							strcpy(shelp, report);
							sprintf(report, "%s %d %s ", shelp, i, command.lab);
							if(!axis) i++;
							else i = MaxAxis +1;
						}while(i <= MaxAxis);
						if(command.code == 0xF1){
							if(para > 0 && para < 8){
								shbyte = 0x01;
								inbyte  = inportb(Reg.IO) | (shbyte << (para-1));
								outportb(Reg.IO, inbyte);
								delay(50);
								inbyte  = inportb(Reg.IO) & ~(shbyte << (para-1));
								outportb(Reg.IO, inbyte);
							}
						}
					}
				*/
					break;
		case 0xE8:  // WA: Wait Absolute
					// No Implementation required
					break;
		case 0xF0:  // WC: Wait for Channel
					// can be interrupted with Esc (NOT LINUX)
					if(para > 0 && para < 8){
						key = 0;
						do{
#ifndef unix
							if(kbhit()) key = getch();
							if(key == 0x1B) break;
#endif
						}while((inportb(Reg.IO) & para) != para);
					}
					break;
		case 0xF2:  // FEP: Find Edge Positive
		case 0xF3:  // FEN: Find Edge Negative
					// can be interrupted with Esc	(NOT LINUX)
					if(axis > 0 && axis <= MaxAxis){
						Select(axis);
						if(command.code == 0xF2) execute(axis, MA, 10000000L, rep);
						else execute(axis, MA, -10000000L, rep);
						execute( axis, TX , 0, rep);
						execute( axis, CLR, 0, rep);
						key = 0;
						do{
#ifndef unix
							if(kbhit()) key = getch();
							if(key == 0x1B) break;
#endif
							delay(60);
						}while((getQMC(0x31) & 0x08) == 0x08);
						if(key != 0x1B){
							delay(200);
							execute(axis, MA, getQMC(0x36), rep);
							sprintf(report," Reference Signal detected ");
						}
						else{
							execute(axis, ST, 0, rep);
							sprintf(report," Edge Find canceled ");
						}
					}
					break;

		default:	sprintf(report," ERROR in [execute]: Command %s not found ",command.lab);
					break;
	  }  // end switch
	} //end else
	return(0);
}

INT Byte2BinStr(BYTE wert, char *s)
{
	BYTE i;

	for(i=0; i < 8; i++){
		if(wert & 0x80) s[i] = '1';
		else s[i] = '0';
		wert <<= 1;
	}
	s[i] = '\0';
	return(0);
}

INT ReadStatus(BYTE axis)
{
	INT ret = FALSE;

	if(axis > 0 && axis < 5){
		Select(axis);
		ret = (int)getQMC(0x31);
	}
	return(ret);
}

INT Read_PES(long *position, long *dyn_pos_err, INT *status)
{
	BYTE i;

	for(i=1; i <= MaxAxis; i++){           /*V31*/
		Select(i);
		position[i]    = getQMC(0x37);
		dyn_pos_err[i] = getQMC(0x60);
		status[i]      = (int)getQMC(0x31);
	}
	return(0);
}


long getQMC(BYTE cmd)
{
	long 	 ret = FALSE;
	long 	 value;
	INT 	 val_int;
	unsigned val_w;

	if(QMCReady()){
		switch(cmd){
			case 0x1D:
			case 0x1E:
			case 0x36:
			case 0x37:
			case 0x3E:
			case 0x4A:
			case 0x4B:
			case 0x4C:
			case 0x57:
			case 0x58:
			case 0x59:  outportb(Reg.Cmd, cmd);
						waitQMC();
						value  = inportb(Reg.Data);  value <<= 8;
						value += inportb(Reg.Data);  value <<= 8;
						waitQMC();
						value += inportb(Reg.Data);  value <<= 8;
						value += inportb(Reg.Data);
						waitQMC();
						ret = value;
						break;
			case 0x4F:
			case 0x30:
			case 0x31:
			case 0x49:
			case 0x50:
			case 0x51:
			case 0x52:
			case 0x53:
			case 0x54:
			case 0x55:
			case 0x56:
			case 0x5A:
			case 0x61:
			case 0x67:
			case 0x6C:
			case 0x6E:	outportb(Reg.Cmd, cmd);
						waitQMC();
						val_w  = inportb(Reg.Data); val_w <<= 8;
						val_w += inportb(Reg.Data) ;
						ret = (long) val_w;
						break;
			case 0x60:
			case 0x2E:  outportb(Reg.Cmd, cmd);
						waitQMC();
						val_int  = inportb(Reg.Data) ; val_int <<= 8;
						val_int += inportb(Reg.Data) ;
						ret = (long) val_int;
						break;
			default:	fprintf(stderr, "ERROR in [getQMC]: wrong command \n");
						break;
		}
	}
	return(ret);
}

INT setQMC(BYTE cmd, long para)
{
	INT ret = FALSE;
	unsigned para_hi;
	unsigned para_lo;
	unsigned csr=0;			// Checksum read
	long	 csc=0;           // Ckecksum calculated
	int	sent=0;
	para_lo = (unsigned)(para & 0xFFFF);
	para_hi = (unsigned)((para >> 16) & 0xFFFF);
	csc     = para_lo + para_hi + cmd;
	//truncate the check sum 
	csc=csc&0xFFFF;
	if(QMCReady()){
		switch(cmd){
			case 0x10:
			case 0x11:
			case 0x12:
			case 0x13:
			case 0x14:
			case 0x16:		outportb(Reg.Cmd, cmd);
						waitQMC();
						outportb(Reg.Data, hi(para_hi));
						waitQMC();
						outportb(Reg.Data, lo(para_hi));
						waitQMC();
						outportb(Reg.Data, hi(para_lo));
						waitQMC();
						outportb(Reg.Data, lo(para_lo));
						waitQMC();
						ret = TRUE;
						sent=4;
						break;
			case 0x15:
			case 0x25:
			case 0x26:
			case 0x27:
			case 0x28:
			case 0x29:
			case 0x2A:
			case 0x2B:
			case 0x2F:
			case 0x32:
			case 0x38:
			case 0x5B:
			case 0x62:
			case 0x66:
			case 0x68:  		outportb(Reg.Cmd, cmd);
						waitQMC();
						outportb(Reg.Data, hi(para_lo));
						waitQMC();
						outportb(Reg.Data, lo(para_lo));
						waitQMC();
						sent=2;
						ret = TRUE;
						break;
			case 0x09:
			case 0x0A:
			case 0x0B:
			case 0x0C:
			case 0x17:
			case 0x18:
			case 0x19:
			case 0x1A:
			case 0x1B:
			case 0x1C:
			case 0x21:
			case 0x22:
			case 0x33:
			case 0x39:
			case 0x3B:
			case 0x3C:
			case 0x3D:
			case 0x3F:
			case 0x40:
			case 0x41:
			case 0x42:
			case 0x43:
			case 0x44:
			case 0x45:
			case 0x46:
			case 0x5C:
			case 0x5D:
			case 0x64:
			case 0x65:
			case 0x6D:
			case 0x70:
			case 0x71: 	outportb(Reg.Cmd, cmd);
						waitQMC();
						sent=0;
						ret = TRUE;
						break;
			default:	fprintf(stderr,"ERROR in [setQMC]: wrong command %0x \n", cmd);
						break;
		}
	} // end of if
	else fprintf(stderr, "ERROR in [sendQMC]: timeout \n");
unsigned csr_lo,csr_hi;
	if(QMCReady()){
		csr_lo  = inportb(Reg.Data); 
		csr_hi = inportb(Reg.Data);
		waitQMC();
		csr=(csr_lo<<8)+(csr_hi);
		if( csr != csc ){
			if(cmd != 0x39){
				ret = FALSE;
				printf("cmd+%d bytes\n",sent);
				fprintf(stderr, "ERROR in [sendQMC]: checksum \n");
fprintf(stderr, "sum(trans)=%x, (command=%x, parahi=%x, paralo=%x, para=%x)\n",csc,cmd,para_hi,para_lo,para);
fprintf(stderr, "checksum=  %x (csr_lo= %x, csr_hi=%x)\n\n\n",csr,csr_lo,csr_hi);
delay(900);

#ifndef unix
				fprintf(stderr, "Press any Key to continue... \n");
				while(!kbhit());
#endif
				}
			}
/*
// checksum error diagnostics
else{
printf("OK\n");
fprintf(stderr, "sum(trans)=%x, (command=%x, parahi=%x, paralo=%x, para=%x)\n",csc,cmd,para_hi,para_lo,para);
fprintf(stderr, "checksum=  %x (csr_lo= %x, csr_hi=%x)\n\n\n",csr,csr_lo,csr_hi);
}
*/
	}
	return(ret);
}

INT Select(BYTE axis)
{
	INT ret = FALSE;

	if( axis > 0 && axis <= MaxAxis){
		if(QMCReady()){
			outportb(Reg.Cmd, axis);
			acta = axis;
			ret = TRUE;
		}
	}
	else fprintf(stderr," ERROR in [Select]: wrong axis identifier\n");
	return(ret);
}

void waitQMC()
{
	if(!QMCReady())
		fprintf(stderr," ERROR in [waitQMC]: Timeout\n");
	return;
}

INT QMCBusy()
{
	INT T_max = 2500;
	INT timeout = 0;

	while( !(StatusBoard() & 0x80) && timeout < T_max) timeout++;
	if(timeout < T_max) return(timeout);
	else return(FALSE);
}

INT QMCReady()
{
	INT T_max = 500;
	INT timeout = 0;

	while( !(StatusBoard() & 0x80) && timeout < T_max) timeout++;
	if(timeout < T_max) return(TRUE);
	else return(FALSE);
}

INT StatusBoard()
{
	return(inport(Reg.Cmd));
}

BYTE lo(unsigned val)
{
	return((BYTE)val);
}

BYTE hi(unsigned val)
{
	return((BYTE) (val >> 8));
}
char *strdst="C Interface by DSt Eisner Consult";
