/*
linux PI stages drivers author matt clark novemeber 1999
based on....

This is the internal header used for the stage library
functions provided by PI

*/
/****************************************************
 *						*
 *   Proj.Name: C - Interface for		*
 *	PI C-842 DC Motor Controller		*
 *						*
 *	Version: 2.1				*
 *   File Name: QFLC300X.H                      *
 ****************************************************
 *						*
 *                                              *
 *	History:		Date:		*
 *	First Implementation	28.11.1995	*
 *                                              *
 ****************************************************/
#ifndef __STAGE_LINUX__
#define __STAGE_LINUX__
#ifdef unix
#define INT short
#endif
#ifndef unix
#define INT int
#endif

//Update, 23.5.01, Steve:
//ioperm is now in <sys/io.h> rather than <asm/io.h>
//since we update the glibc libraries.
#include <sys/io.h>    

#define TRUE  1
#define FALSE 0

#define BYTE unsigned char
#ifdef _LINUX_TYPES_H_
 #define BYTE uint_t
#endif

#define LIMITMODE	(unsigned char)(255)
#define BOARDMODE	(unsigned char)(9)
#define CMD_NOPARA	1

#define RD_NOFRAC	2
#define RD_U1616	4
#define RD_S1616	5
#define RD_U0032	6

#define WR_NOFRAC	12
#define WR_U1616	14
#define WR_S1616	15
#define WR_U0032	16
#define EXDEF		20

#define NOPARAM 0

#define MAXAXIS	4

extern "C"  {

struct REG{              // default Addresses
	unsigned long	Data ;
	unsigned long	Cmd  ;
	unsigned long	Func ;
	unsigned long	IO	 ;
};

struct TCMD{
	BYTE 	code;
	BYTE	sect;
	char	lab[5];
};

// functions for external access

void Init_LS(BYTE ax1, BYTE ax2, BYTE ax3, BYTE ax4);
INT  Board_Installed(void);
INT  InitBoard(BYTE mode);
void InitAxis(BYTE axis);
void translate(char *comstr, char *report);
INT  execute(BYTE axis, TCMD command, long para, char *report);
void ClearAxisStatus(BYTE axis);
INT  Select(BYTE axis);
INT  setQMC(BYTE cmd, long para);
long getQMC(BYTE cmd);
INT  QMCbusy(void);
INT  QMCready(void);
INT  Read_PES(long *position, long *dyn_pos_err, INT *status);

void Set_BaseAddress(unsigned long address);
BYTE Axis_Installed(void);
BYTE moving(BYTE axis);

// internal functions
void waitQMC(void);
INT QMCBusy(void);
INT QMCReady(void);
INT StatusBoard(void);
BYTE lo(unsigned val);
BYTE hi(unsigned val);
INT Byte2BinStr(BYTE wert, char *s);

//Definition of Commands

TCMD
	UPDATE ={0x1A, CMD_NOPARA,  "UP:" },
	TP ={0x37, RD_NOFRAC,   "TP:" },
	GLS={0x67, RD_NOFRAC,   "GLS:"},
	TT ={0x4A, RD_NOFRAC,   "TT:" },
	MUP={0x5B, WR_NOFRAC,   "MUP:"},
	TS ={0x31, RD_NOFRAC,	"TS:" },
	GIP={0x36, RD_NOFRAC,	"GIP:"},
	MA ={0x10, EXDEF,		"MA:" },
	SV ={0x11, WR_U1616,	"SV:" },
//10
	GMO={0x49, RD_NOFRAC,	"GMO:"},
	MR ={0xE1, EXDEF,		"MR:" },
	GH ={0xE2, EXDEF,		"GH:" },
	TX ={0x36, RD_NOFRAC,	"TX:" },
	WS ={0xE3, EXDEF,		"WS:" },
	MN ={0x43, CMD_NOPARA,	"MN:" },
	MF ={0x42, CMD_NOPARA,	"MF:" },
	TV ={0xE4, EXDEF,		"TV:" },
	CLR={0x33, CMD_NOPARA,	"CLR:"},
	ST ={0xE5, EXDEF,		"ST:" },
//20
	SA ={0x12, WR_U1616,	"SA:" },
	DH ={0x3F, CMD_NOPARA,	"DH:" },
	TY ={0x4B, RD_U1616,	"TY:" },
	GPP={0x1D, RD_NOFRAC,	"GPP:"},
	GPV={0x1E, RD_S1616,	"GPV:"},
	TF ={0x60, RD_NOFRAC,	"TF:" },
	RT ={0x39, CMD_NOPARA,	"RT:" },
	GB ={0x57, RD_NOFRAC,	"GB:" },
	GIR={0x30, RD_NOFRAC,	"GIR:"},
	SAN={0x5C, CMD_NOPARA,	"SAN:"},
//30
	SAF={0x5D, CMD_NOPARA,	"SAF:"},
	TL ={0x4C, RD_U1616,	"TL:" },
	ASN={0x45, CMD_NOPARA,	"ASN:"},
	ASF={0x44, CMD_NOPARA,	"ASF:"},
	SMO={0x62, WR_NOFRAC,	"SMO:"},
	TE ={0xE6, EXDEF,		"TE:" },
	SFP={0x21, CMD_NOPARA,	"SFP:"},
	SFV={0x22, CMD_NOPARA,	"SFV:"},
	DP ={0x25, WR_NOFRAC,	"DP:" },
	DD ={0x27, WR_NOFRAC,	"DD:" },
//40
	DI ={0x26, WR_NOFRAC,	"DI:" },
	DV ={0x2A, WR_NOFRAC,	"DV:" },
	DF ={0x2B, WR_NOFRAC,	"DF:" },
	DL ={0x28, WR_NOFRAC,	"DL:" },
	GP ={0x50, RD_NOFRAC,	"GP:" },
	GD ={0x52, RD_NOFRAC,	"GD:" },
	GI ={0x51, RD_NOFRAC,	"GI:" },
	GV ={0x5A, RD_NOFRAC,	"GV:" },
	GF ={0x54, RD_NOFRAC,	"GF:" },
	GL ={0x53, RD_NOFRAC,	"GL:" },
//50
	GPE={0x55, RD_NOFRAC,	"GPE:"},
	GSI={0x2E, RD_NOFRAC,	"GSI:"},
	RTI={0x32, WR_NOFRAC,	"RTI:"},
	GIM={0x56, RD_NOFRAC,	"GIM:"},
	TG ={0xE7, EXDEF,		"TG:" },
	SCN={0x68, WR_NOFRAC,	"SCN:"},
	SCI={0x64, CMD_NOPARA,	"SCI:"},
	LN ={0x70, CMD_NOPARA,	"LN: "},
	LF ={0x71, CMD_NOPARA,	"LF: "},
	SPS={0x0B, CMD_NOPARA,	"SPS:"},
//60
	SPT={0x09, CMD_NOPARA,	"SPT:"},
        SMA={0x15, WR_NOFRAC,	"SMA:"},
        GMA={0x4F, RD_NOFRAC,	"GMA:"},
	CLP={0x46, CMD_NOPARA,	"CLP:"},
	SYN={0x47, CMD_NOPARA,	"SYN:"},
	SPE={0x29, WR_NOFRAC,	"SPE:"},
	SCH={0x65, CMD_NOPARA,	"SCH:"},
	SLS={0x66, WR_NOFRAC,	"SLS:"},
	VE ={0x6C, RD_NOFRAC,	"VE: "},
	GTI={0x3E, RD_NOFRAC,	"GTI:"},
//70
	WA ={0xE8, EXDEF,		"WA: "},
	SBT={0x17, CMD_NOPARA,	"SBT:"},
	SBP={0x18, CMD_NOPARA,	"SBP:"},
	SBN={0x19, CMD_NOPARA,	"SBN:"},
	SBA={0x1B, CMD_NOPARA,	"SBA:"},
	SBC={0x1C, CMD_NOPARA,	"SBC:"},
	SBF={0x6D, CMD_NOPARA,	"SBF:"},
	SB ={0x16, WR_NOFRAC,	"SB:" },
	SIM={0x2F, WR_NOFRAC,	"SIM:"},
	GJ ={0x58, RD_U0032,	"GJ:" },
//80
	SJ ={0x13, WR_U0032,	"SJ:" },
	SRA={0x14, WR_S1616,	"SRA:"},
	GRA={0x59, RD_S1616,	"GRA:"},
	SPV={0x0A, CMD_NOPARA,	"SPV:"},
	SPG={0x0C, CMD_NOPARA,	"SPG:"},
	SL ={0xE9, EXDEF,		"SL: "},
	AN ={0x41, CMD_NOPARA,	"AN: "},
	AF ={0x40, CMD_NOPARA,	"AF: "},
	SOP={0x3C, CMD_NOPARA,	"SOP:"},
	SOT={0x3D, CMD_NOPARA,	"SOT:"},
//90
	SOH={0x3B, CMD_NOPARA,	"SOH:"},
	SST={0x38, WR_NOFRAC,	"SST:"},
	GT ={0x61, RD_NOFRAC,	"GT: "},
	GOM={0x6E, RD_NOFRAC,	"GOM:"},
	TZ ={0xEA, EXDEF,		"TZ: "},
	TC ={0xEB, EXDEF,		"TC: "},
	CN ={0xEC, EXDEF,		"CN: "},
	CF ={0xED, EXDEF,		"CF: "},
	IV ={0xEE, EXDEF,		"IV: "},
	AB ={0xEF, EXDEF,		"AB: "},
//100
	WC ={0xF0, EXDEF,		"WC: "},
	WT ={0xF1, EXDEF,		"WT: "},
	FEP={0xF2, EXDEF,		"DEP:"},
	FEN={0xF3, EXDEF,		"FEN:"};

	}
#endif
