/*
linux PI stages drivers author matt clark novemeber 1999
based on....


This is the external header for including in programs
that use the linux version of the PI library

*/

/****************************************************
 *						*
 *   Proj.Name: C - Interface for		*
 *	PI C-842 DC Motor Controller		*
 *						*
 *	Version: 2.1				*
 *   File Name: QFLC300X.H                      *
 ****************************************************
 *						*
 *                                              *
 *	History:		Date:		*
 *	First Implementation	28.11.1995	*
 *                                              *
 ****************************************************/
#ifndef __STAGE_LINUX__
#define __STAGE_LINUX__


//Update, 23.5.01, Steve
//ioperm is now in <sys/io.h> rather than <asm/io.h
//since we update the glibc libraries
#include <sys/io.h>    

#define TRUE  1
#define FALSE 0

#define BYTE unsigned char
#ifdef _LINUX_TYPES_H_
 #define BYTE uint_t
#endif

#define LIMITMODE	(unsigned char)(255)
#define BOARDMODE	(unsigned char)(9)
#define CMD_NOPARA	1

#define RD_NOFRAC	2
#define RD_U1616	4
#define RD_S1616	5
#define RD_U0032	6

#define WR_NOFRAC	12
#define WR_U1616	14
#define WR_S1616	15
#define WR_U0032	16
#define EXDEF		20

#define NOPARAM 0

#define MAXAXIS	4

extern "C"  {

struct REG{              // default Addresses
	unsigned	Data ;
	unsigned	Cmd  ;
	unsigned	Func ;
	unsigned	IO	 ;
};

struct TCMD{
	BYTE 	code;
	BYTE	sect;
	char	lab[5];
};

// functions for external access

void Init_LS(BYTE ax1, BYTE ax2, BYTE ax3, BYTE ax4);
int  Board_Installed(void);
int  InitBoard(BYTE mode);
void InitAxis(BYTE axis);
void translate(char *comstr, char *report);
int  execute(BYTE axis, TCMD command, long para, char *report);
void ClearAxisStatus(BYTE axis);
//int  select(BYTE axis); // conflicts with stdlibrary type and is boring
int  setQMC(BYTE cmd, long para);
long getQMC(BYTE cmd);
int  QMCbusy(void);
int  QMCready(void);
int  Read_PES(long *position, long *dyn_pos_err, int *status);

void Set_BaseAddress(unsigned address);
BYTE Axis_Installed(void);
BYTE moving(BYTE axis);

// internal functions
void waitQMC(void);
int QMCBusy(void);
int QMCReady(void);
int StatusBoard(void);
BYTE lo(unsigned val);
BYTE hi(unsigned val);
int Byte2BinStr(BYTE wert, char *s);

//Definition of Commands

extern TCMD UPDATE ;
extern TCMD TP ;
extern TCMD GLS;
extern TCMD TT ;
extern TCMD MUP;
extern TCMD TS ;
extern TCMD GIP;
extern TCMD MA ;
extern TCMD SV ;
//10
extern TCMD GMO;
extern TCMD MR ;
extern TCMD GH ;
extern TCMD TX ;
extern TCMD WS ;
extern TCMD MN ;
extern TCMD MF ;
extern TCMD TV ;
extern TCMD CLR;
extern TCMD ST ;
//20
extern TCMD SA ;
extern TCMD DH ;
extern TCMD TY ;
extern TCMD GPP;
extern TCMD GPV;
extern TCMD TF ;
extern TCMD RT ;
extern TCMD GB ;
extern TCMD GIR;
extern TCMD SAN;
//30
extern TCMD SAF;
extern TCMD TL ;
extern TCMD ASN;
extern TCMD ASF;
extern TCMD SMO;
extern TCMD TE ;
extern TCMD SFP;
extern TCMD SFV;
extern TCMD DP ;
extern TCMD DD ;
//40
extern TCMD DI ;
extern TCMD DV ;
extern TCMD DF ;
extern TCMD DL ;
extern TCMD GP ;
extern TCMD GD ;
extern TCMD GI ;
extern TCMD GV ;
extern TCMD GF ;
extern TCMD GL ;
//50
extern TCMD GPE;
extern TCMD GSI;
extern TCMD RTI;
extern TCMD GIM;
extern TCMD TG ;
extern TCMD SCN;
extern TCMD SCI;
extern TCMD LN ;
extern TCMD LF ;
extern TCMD SPS;
//60
extern TCMD SPT;
extern TCMD SMA;
extern TCMD GMA;
extern TCMD CLP;
extern TCMD SYN;
extern TCMD SPE;
extern TCMD SCH;
extern TCMD SLS;
extern TCMD VE ;
extern TCMD GTI;
//70
extern TCMD WA ;
extern TCMD SBT;
extern TCMD SBP;
extern TCMD SBN;
extern TCMD SBA;
extern TCMD SBC;
extern TCMD SBF;
extern TCMD SB ;
extern TCMD SIM;
extern TCMD GJ ;
//80
extern TCMD SJ ;
extern TCMD SRA;
extern TCMD GRA;
extern TCMD SPV;
extern TCMD SPG;
extern TCMD SL ;
extern TCMD AN ;
extern TCMD AF ;
extern TCMD SOP;
extern TCMD SOT;
//90
extern TCMD SOH;
extern TCMD SST;
extern TCMD GT ;
extern TCMD GOM;
extern TCMD TZ ;
extern TCMD TC ;
extern TCMD CN ;
extern TCMD CF ;
extern TCMD IV ;
extern TCMD AB ;
//100
extern TCMD WC ;
extern TCMD WT ;
extern TCMD FEP;
extern TCMD FEN;

}
#endif
