#include "s_main.h"
#include <string.h>


// private variables for this library

BYTE    PI_LIMIT_SENSE=64+128;
float   PI_CPM[5];
float   PI_MPC[5];
float   PI_STAGE_OFFSET[5];
BOOL    PI_SETUP_STAGE=FALSE;
BOOL    PI_SHUTDOWN=FALSE;
BOOL    PI_STAGE_ERROR=FALSE;  

void PI_catch_signal(int sig){
        cout<<"PI:Signal "<<sig<<" caught \n";
        cout<<"PI:Possibly SIGINT =2 (interrupt from keyboard)\n";
        cout<<"PI:         SIGQUIT=3 (quit from keyboard)\n";
        cout<<"PI:This signal will not be caught again, \n";
        cout<<"PI:but leave a few seconds to see if the is a soft landing\n";
        cout<<"PI:Calling PI_stop_dead()\n";
        PI_stop_dead();
	cout<<"PI:Calling PI_close_stage()\n";
	PI_close_stage();
	cout<<"PI:Quitting\n";
	exit(3);
	}

void PI_init_limits(int a,int b,int c,int d){
	Init_LS(a,b,c,d);
	PI_LIMIT_SENSE=(BYTE)((1+2)*a+(4+8)*b+(16+32)*c+(64+128)*d);

// from values passed to this fn, work out new and old stages
// and calculate counts/micron accordingly

	PI_CPM[0]=PI_CPM_O;PI_MPC[0]=PI_MPC_O;//pos zero isn't used but it doesn't hurt to set it....
	if(a==0){ 	PI_CPM[1]=PI_CPM_O;PI_MPC[1]=PI_MPC_O; }
	else{ 		PI_CPM[1]=PI_CPM_N;PI_MPC[1]=PI_MPC_N; }

	if(b==0){ 	PI_CPM[2]=PI_CPM_O;PI_MPC[2]=PI_MPC_O; }
	else{ 		PI_CPM[2]=PI_CPM_N;PI_MPC[2]=PI_MPC_N; }

	if(c==0){ 	PI_CPM[3]=PI_CPM_O;PI_MPC[3]=PI_MPC_O; }
	else{ 		PI_CPM[3]=PI_CPM_N;PI_MPC[3]=PI_MPC_N; }

	if(d==0){ 	PI_CPM[4]=PI_CPM_O;PI_MPC[4]=PI_MPC_O; }
	else{ 		PI_CPM[4]=PI_CPM_N;PI_MPC[4]=PI_MPC_N; }

	}

BYTE PI_get_limit_status(void){
char 	report[PI_REPORT_LENGTH];
BYTE 	value;
int 	i=0;
	memset(report,PI_REPORT_LENGTH,'\0');
	execute(0,GLS,0,report);
	while(report[i]!=':'&&i<PI_REPORT_LENGTH)i++;
	value=(atoi(report+i+1));
//	cout<<"\n\r******************\n\r";
//	cout<<"S_MAIN:get_limit_status: value="<<((int)value);
	value=value^PI_LIMIT_SENSE;
//	cout << "(" << (int) value << ")" << endl;
	report[PI_REPORT_LENGTH-1]='\0';
//	cout<<"S_MAIN: get_limit_status: report=\""<<report<<"\""<<endl;
	return value;
	}

BOOL PI_moving(AXIS axis){
char report[PI_REPORT_LENGTH];
//NOTE: this function DOES work whereas using the function
//'moving(axis)' supplied with the libraries DOES NOT
//nor does get_status and checking the 'motion complete' flag
//both of these techniques are garbage. matt 31/3/98
	execute((BYTE) axis,TV,0,report);
	if(report[8]!='0') return TRUE;
	return FALSE;
	}

void PI_pstat(AXIS axis){
int 	s;
char 	report[PI_REPORT_LENGTH];

	execute((BYTE) axis,TS,0,report);
	int i=0;
	while(report[i]!=':'&&i<PI_REPORT_LENGTH){i++;}
	i++;
	s=atoi(report+i);

	cout<<"bit 0  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15"<<endl;
	cout<<"    "<<PI_gtz((s&1u))<<"  "<<PI_gtz((s&2u))<<"  "<<PI_gtz((s&4u)) <<"  "<< PI_gtz((s&8u))<<
		"  "<<PI_gtz((s&16u))<<"  "<<PI_gtz((s&32u)) <<"  "<<PI_gtz((s&64u))<<"  "<<PI_gtz((s&128u))<<
		"  "<<PI_gtz((s&256u))<<"  "<<PI_gtz((s&512u))<<"  "<<PI_gtz((s&1024u))<<"  "<<PI_gtz((s&2048u))<<
		"  "<<PI_gtz((s&4096u))<<"  "<<PI_gtz((s&8192u))<<"  "<<PI_gtz((s&16384u))<<"  "<<PI_gtz((s&32768u))<<endl;
}

BOOL PI_axis_is_on(AXIS axis) {
int s;
char report[PI_REPORT_LENGTH];
	execute((BYTE) axis,TS,0,report);
	int i=0;
	while(report[i]!=':'&&i<PI_REPORT_LENGTH){i++;}
	i++;
	s=atoi(report+i);

	if(PI_gtz((s&512u))==0) return FALSE;
	else return TRUE;
	}


int PI_gtz(int num){
	if(num>0) return 1;
	else return 0;
	}


// modified by matt Feb 2000 to double check the stopping...
void PI_wait_axis_stop(AXIS axis){
	//check once....
	do{
		if(PI_limit_ok()!=TRUE){
			PI_stop_dead();
			PI_STAGE_ERROR=TRUE;
			cout<<"\a\ain PI_wait_axis_stop 123 Limit switch hit - stage stopped\n";
			cout<<"Limit status ="<<PI_limit_ok()<<endl;
			PI_move_off_limits();
			}
		usleep(PI_SAMPLE_WAIT);//don't check more than 1000Hz, its not worth it
		}while(PI_moving(axis)!=0);

	usleep(10*PI_SAMPLE_WAIT);// Sleep for a bit to allow the stages to get going again

	// check again. (to make sure, sometimes the stages can <stop> as they settle so to prevent a false alarm)...
	do{
		if(PI_limit_ok()!=TRUE){
			PI_stop_dead();
			PI_STAGE_ERROR=TRUE;
			cout<<"\a\ain PI_wait_axis_stop 137 Limit switch hit - stage stopped\n";
			cout<<"Limit status ="<<PI_limit_ok()<<endl;
			PI_move_off_limits();
			}
		usleep(PI_SAMPLE_WAIT);//don't check more than 1000Hz, its not worth it
		}while(PI_moving(axis)!=0);
	}

void PI_wait_all_stop(void){
	//check once....
	do{
		if(PI_limit_ok()!=TRUE){
			PI_stop_dead();
			PI_STAGE_ERROR=TRUE;
			cout<<"\a\ain PI_wait_all_stop 151 Limit switch hit - stage stopped\n";
			cout<<"Limit status ="<<PI_limit_ok()<<endl;
			PI_move_off_limits();
			}
		usleep(1000);//don't check more than 1000Hz, its not worth it
		}while(PI_any_axis_moving()!=0);
	        usleep(10*PI_SAMPLE_WAIT);// Sleep for a bit to allow the stages to get going again      

	// check again. (to make sure, sometimes the stages can <stop> as they settle so to prevent a false alarm)...
	do{
		if(PI_limit_ok()!=TRUE){
			PI_stop_dead();
			PI_STAGE_ERROR=TRUE;
			cout<<"\a\ain PI_wait_all_stop 164 Limit switch hit - stage stopped\n";
			cout<<"Limit status ="<<PI_limit_ok()<<endl;
			PI_move_off_limits();
			}
		usleep(1000);//don't check more than 1000Hz, its not worth it
		}while(PI_any_axis_moving()!=0);
	}


BYTE PI_any_axis_moving(void){
	return PI_moving(1)+PI_moving(2)+PI_moving(3)+PI_moving(4);
	}

void PI_stop_dead(void)
{
// works by finding the positions of all the axes at time of calling this
// function, and then telling the stages to go there. This function replaces
// an old version of the function, which used the "AB" command, which is
// a software-emulated command that only works properly on axes 1,2 and 3.

	char report[PI_REPORT_LENGTH];
	int i,j;
	int position[5];

	execute(0,TP,0,report); // get the position
	j=0;
	for(i=1;i<5;i++){
		while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
		j++;
		position[i]=(atol(report+j));
	}

	for(i=1;i<5;i++){
		execute(i,MA,position[i],report); // set desired pos to current pos
	}
}



void PI_set_origin(void){
	PI_set_origin(1);
	PI_set_origin(2);
	PI_set_origin(3);
	PI_set_origin(4);
	}

void PI_set_origin(AXIS axis){
POS	pos,err;
int	j=0;
char 	report[PI_REPORT_LENGTH];
	execute(axis,DH,0,report);
	//get the unoffsetted position.....
        execute(axis,TP,0,report); // get the position
        while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
        pos=atof(report+j+1)*PI_MPC[axis];   
	PI_STAGE_OFFSET[axis]=-pos;
	pos=PI_get_position(axis);
	}

void PI_recall_origin(void){
	PI_recall_origin(PI_ORIGIN_FILE_NAME);
	}

void PI_recall_origin(char *name){
ifstream is;
BOOL	ok=TRUE;
char	temp[100];
	if(access(name,R_OK)!=0){
		cout<<"Unable to open file "<<name<<" (reason access denied/ file doesn't exist?)"<<endl;;
		ok=FALSE;
		}
	else{
	if(access(name,W_OK)!=0){
		cout<<"Caution, stage origin file does not have write permission"<<endl;
		}
	else{
	is.open(name);
	if(is.bad()){
		cout<<"Unable to open file "<<name<<" (reason stream is bad)"<<endl;
		ok=FALSE;
		}
	else{
		cout<<"Opening stage origin file... restoring origin\n";
		is>>PI_STAGE_OFFSET[1];
		is>>PI_STAGE_OFFSET[2];
		is>>PI_STAGE_OFFSET[3];
		is>>PI_STAGE_OFFSET[4];
		cout<<"Using ("<<PI_STAGE_OFFSET[1]<<","<<PI_STAGE_OFFSET[2]<<","<<PI_STAGE_OFFSET[3]<<","<<PI_STAGE_OFFSET[4]<<")"<<endl;
		}
		}	
		}	
	if(ok==FALSE){
		cout<<"Unable to read the stage origin, would you like to\n";
		do{
			cout<<"use 0,0,0,0 <z> or <a>bort and correct the situation ?";
			cin>>temp;
			}
			while((temp[0]!='a')&&(temp[0]!='z'));
		if(temp[0]=='a')exit(1);
		PI_STAGE_OFFSET[1]=0;
		PI_STAGE_OFFSET[2]=0;
		PI_STAGE_OFFSET[3]=0;
		PI_STAGE_OFFSET[4]=0;
		}
	}	

void PI_save_origin(void){
	PI_save_origin(PI_ORIGIN_FILE_NAME);
	}

void PI_save_origin(char *name){
POS	pos[5],err[5];
ofstream	os;
	// MAKE SURE that the stages have stopped...
	// check once...
	PI_wait_all_stop();
	// pause..
	usleep(100000);
	// check again.
	PI_wait_all_stop();

	PI_get_position_and_error(pos,err);
	if(access(name,W_OK)!=0){
		cout<<"Caution unable to save the stage origin position\n";
		cout<<"Reason access denied"<<endl;
		cout<<"Currently ("<<PI_STAGE_OFFSET[1]<<","<<PI_STAGE_OFFSET[2]<<","<<PI_STAGE_OFFSET[3]<<","<<PI_STAGE_OFFSET[4]<<")"<<endl;
		cout<<"Current position ("<<pos[1]<<","<<pos[2]<<","<<pos[3]<<","<<pos[4]<<")"<<endl;
		cout<<"You can use these values to reset the origin if you wish\n";
		}
	else{
	
		os.open(name);
		os<<pos[1]<<"\n";		
		os<<pos[2]<<"\n";		
		os<<pos[3]<<"\n";		
		os<<pos[4]<<"\n";		
		cout<<"Saving position as ("<<pos[1]<<","<<pos[2]<<","<<pos[3]<<","<<pos[4]<<")\n\r";
		}
	}
void PI_close_stage(void){
	if(PI_SHUTDOWN==TRUE)return;
	PI_SETUP_STAGE=FALSE;
	PI_SHUTDOWN=TRUE;
	PI_save_origin();
	cout<<"Releasing the stage lock\n\r";
	unlink(PI_LOCKFILE);
	}

void PI_setup_stage(int add){
char 	report[PI_REPORT_LENGTH];
ofstream os;
int	tmp;
pid_t	id;
	if(PI_SETUP_STAGE==TRUE){
//		cout<<"PI_SETUP: ignoring call\n";
		return;
		}
//	cout<<"PI_SETUP: attempting to get a lock on the stages... ";
	if(access(PI_LOCKFILE,F_OK)==0){
		cout<<"Lock file found in "<<PI_LOCKFILE<<"\n";
		cout<<"Stages previosly locked, look in file for the process id\n";
		cout<<"Cannot open stages, quitting\n";
		exit(2);
		}
	if(access(PI_LOCKFILEDIR,W_OK)!=0){
		cout<<"No lock file found (good)\n";
		cout<<"But unable to write to directory "<<PI_LOCKFILEDIR<<" (bad)\n";
		cout<<"Cannot lock stages, quitting\n";
		exit(3);
		}
	os.open(PI_LOCKFILE);
	id=getpid();
	os<<"Locked by process "<<(int)id<<"\n";
	os.close();
//	cout<<"Stages locked successfully\n";
//	cout<<"Setting default signal handlers\n";
	signal(SIGQUIT,PI_catch_signal);
        signal(SIGINT,PI_catch_signal);  	
//	cout<<"Calling Set_BaseAddress()"<<endl;
	Set_BaseAddress(add);
//	cout<<"Calling InitBoard()"<<endl;
//	cout<<"Result of InitBoard ";
	tmp=InitBoard(0x9);
//	cout<<endl;
//	cout<<"Calling InitAxis()"<<endl;
	InitAxis(1);
	InitAxis(2);
	InitAxis(3);
	InitAxis(4);
//	cout<<"Calling init_limits()"<<endl;
//	PI_init_limits(0,0,0,1); // axes 1 and 2 are old stages (old setup)
//	PI_init_limits(0,0,1,1); // axes 1 and 2 are old stages and we have 2 stages only
	PI_init_limits(1,1,0,1); // axes 1 and 2 are new stages (new setup)
//	cout<<"Calling  execute(0,LN,0,report)"<<endl;
	execute(0,LN,0,report);
	PI_SETUP_STAGE=TRUE;
	PI_SHUTDOWN=FALSE;
//	cout<<"Calling set origin\n";
	PI_set_origin();
//	cout<<"Recalling the saved origin\n";
	PI_recall_origin();
	}


inline BOOL PI_limit_ok(void){
	return (PI_get_limit_status()==(unsigned int)255);
	}

void PI_move_off_limits(void){
	while(PI_limit_ok()!=TRUE){
		PI_move_off_limit_switches();
	        do{
       			usleep(1000);
                	}while(PI_any_axis_moving()!=0);        
		}
		PI_STAGE_ERROR=FALSE;
		}

void PI_move_off_limit_switches(void){
char 	report[PI_REPORT_LENGTH];
char  	limit_status;

	limit_status=PI_get_limit_status();
	limit_status=255-limit_status;
//stage number 1
	if(limit_status&1)	execute(1,MR,-SW_MV,report);
	if(limit_status&2)	execute(1,MR,SW_MV,report);
//stage number 2 
	if(limit_status&4)	execute(2,MR,-SW_MV,report);
	if(limit_status&8)	execute(2,MR,SW_MV,report);
//stage number 3
	if(limit_status&16)	execute(3,MR,-SW_MV,report);
	if(limit_status&32)	execute(3,MR,SW_MV,report);
//stage number 4
	if(limit_status&64)	execute(4,MR,-SW_MV,report);
	if(limit_status&128)	execute(4,MR,SW_MV,report);
	}


void PI_moveto(POS x,POS y,POS z,POS n,BOOL wait){
	if(x!=PI_NO_MOVEMENT)PI_moveto(1,x,PI_NO_WAIT);
	if(y!=PI_NO_MOVEMENT)PI_moveto(2,y,PI_NO_WAIT);
	if(z!=PI_NO_MOVEMENT)PI_moveto(3,z,PI_NO_WAIT);
	if(n!=PI_NO_MOVEMENT)PI_moveto(4,n,PI_NO_WAIT);
	PI_wait_all_stop();
	}

void PI_moveto(AXIS axis,POS x,BOOL wait){
char  report[PI_REPORT_LENGTH];
int 	x_in_counts;

	x=x-PI_STAGE_OFFSET[axis];
	x_in_counts=(long)(x*PI_CPM[axis]);
	if(PI_STAGE_ERROR==TRUE){
		cout<<"moveto command blocked by stage error\n";
		return;
		}
	execute((char) axis,MA,x_in_counts,report);
	if(wait==PI_NO_WAIT)return;
	do{
		if(PI_limit_ok()!=TRUE){
			PI_stop_dead();
			PI_STAGE_ERROR=TRUE;
			cout<<"\a\ain PI_moveto 422 Limit switch hit - stage stopped\n";
			cout<<"Limit status ="<<PI_limit_ok()<<endl;
			PI_move_off_limits();
			}
		}while(PI_moving(axis)!=0);

// This next bit added by Steve 1st March 2000 to try and REALLY wait for the stages to stop moving 
// (rather than just start to stop) 
// --------- START OF NEW BIT ---------       
        usleep(10*PI_SAMPLE_WAIT);// Sleep for a bit to allow the stages to get going again 

        // check again. (to make sure, sometimes the stages can <stop> as they settle so to prevent a
	// false alarm)...
        do{  
                if(PI_limit_ok()!=TRUE){
                        PI_stop_dead(); 
                        PI_STAGE_ERROR=TRUE;
                        cout<<"\a\ain PI_moveto 439 Limit switch hit - stage stopped\n";
                        cout<<"Limit status ="<<PI_limit_ok()<<endl;     
                        PI_move_off_limits();
                        }
                usleep(PI_SAMPLE_WAIT);//don't check more than 1000Hz, its not worth it
                }while(PI_moving(axis)!=0);  

// --------- END OF NEW BIT ---------
	}

void PI_shift(POS x,POS y,POS z,POS n,BOOL wait){
	if(PI_STAGE_ERROR==TRUE){
		cout<<"shift command blocked by stage error\n";
		return;
		}
	if(x!=PI_NO_MOVEMENT)PI_shift(1,x,PI_NO_WAIT);
	if(y!=PI_NO_MOVEMENT)PI_shift(2,y,PI_NO_WAIT);
	if(z!=PI_NO_MOVEMENT)PI_shift(3,z,PI_NO_WAIT);
	if(n!=PI_NO_MOVEMENT)PI_shift(4,n,PI_NO_WAIT); 
	PI_wait_all_stop();
	}

void PI_shift(AXIS axis,POS x,BOOL wait){
char  	report[PI_REPORT_LENGTH];
int 	x_in_counts;
	
	x_in_counts=(long)(x*PI_CPM[axis]);
	if(PI_STAGE_ERROR==TRUE){
		cout<<"shift command blocked by stage error\n";
		return;
		}
	execute((char) axis,MR,x_in_counts,report);
	if(wait==PI_NO_WAIT)return;
	do{
		if(PI_limit_ok()!=TRUE){
			PI_stop_dead();
			PI_STAGE_ERROR=TRUE;
			cout<<"in PI_shift 476 Limit switch hit stage stopped\n";
			cout<<"Limit status ="<<PI_limit_ok()<<endl;
			PI_move_off_limits();
			}
		}while(PI_moving(axis)!=0);

// This next bit added by Steve 1st March 2000 to try and REALLY wait for the stages to stop moving 
// (rather than just start to stop) 
// --------- START OF NEW BIT ---------       
        usleep(10*PI_SAMPLE_WAIT);// Sleep for a bit to allow the stages to get going again 

        // check again. (to make sure, sometimes the stages can <stop> as they settle so to prevent a
	// false alarm)...
        do{  
                if(PI_limit_ok()!=TRUE){
                        PI_stop_dead(); 
                        PI_STAGE_ERROR=TRUE;
                        cout<<"\a\ain PI_shift 493 Limit switch hit - stage stopped\n";
                        cout<<"Limit status ="<<PI_limit_ok()<<endl;     
                        PI_move_off_limits();
                        }
                usleep(PI_SAMPLE_WAIT);//don't check more than 1000Hz, its not worth it
                }while(PI_moving(axis)!=0);  

// --------- END OF NEW BIT ---------

	}



void PI_set_velocity(AXIS axis, float velocity){
int 	i;
char 	report[PI_REPORT_LENGTH];
int 	velocity_in_counts;

 	velocity_in_counts=(int)(velocity*PI_CPM[axis]);
	if(axis==0){
		for(i=1;i<5;i++){
			execute(i,SV,(int)(velocity*PI_CPM[i]),report);
			}
		}
	else{
		execute((BYTE) axis,SV,(int) velocity_in_counts,report);
		}
	}

// get the position functions.......
// a little tricky here, usually use the PI_get_position(POS *pos) or PI_get_position(AXIS),
// these return the value you are looking for (where the stage is), 
// HOWEVER, the stage doesn't always stop exactly where it should, just very very near so 
// if you need absolute accuracy (for save and recall origin for instance) use
// PI_get_position(POS *pos,POS *err) and PI_get_position(AXIS,POS *pos,POS *err)
// the place you wanted to go P_desired=P_absolute+P_error=(the simple functions)
// the actual place the stage is at is P_absolute, P_error is very very small so for
// all usual purposes the simple function which return pos+err are fine.

// Also note that the position is just the position of the encoder, the error should be
// smaller than the physical stage error and thus within spec... matt December 1999

// Simple functions for determining where the stages are
void PI_get_position(POS *position){
int	i;
POS 	pos[5],err[5];
	PI_get_position_and_error(pos, err);
	for(i=1;i<5;i++){
		position[i]=pos[i]-err[i];
		}
}

POS PI_get_position(AXIS axis) {
POS pos,err;
	PI_get_position_and_error(axis,&pos,&err);
	return pos+err;
	}

float PI_get_position_NOW(AXIS axis){
char    report[PI_REPORT_LENGTH];
int     j=0;
// if the axis is set to zero 0 then it does all axis
        execute(axis,TP,0,report); // get the position
        while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
        return ((atof(report+j+1)*PI_MPC[axis])+PI_STAGE_OFFSET[axis]);
        }
                
//Complicated functions for determining the position of the stages with no error
void PI_get_position_and_error(POS *position, POS *pos_err) {
// this fn replaces Read_PES supplied with the stage. Unlike Read_PES,
// THIS fn actaully works on all 4 axes....

char 	report[PI_REPORT_LENGTH];
int 	axis,j=0;
// if the axis is set to zero 0 then it does all axis
	execute(0,TP,0,report); // get the position
	for(axis=1;axis<5;axis++){
		while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
		j++;
		position[axis]=(atof(report+j)*PI_MPC[axis])+PI_STAGE_OFFSET[axis];
		}

	j=0;
	execute(0,TF,0,report); // get the position error
	for(axis=1;axis<5;axis++){
		while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
		j++;
		pos_err[axis]=(atol(report+j)*PI_MPC[axis]);
		}
	}


void PI_get_position_and_error(AXIS axis,POS *position, POS *pos_err){
char 	report[PI_REPORT_LENGTH];
int 	j=0;
// if the axis is set to zero 0 then it does all axis
	execute(axis,TP,0,report); // get the position
	while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
	*position=(atof(report+j+1)*PI_MPC[axis])+PI_STAGE_OFFSET[axis];
	j=0;
	execute(axis,TF,0,report); // get the position error
	while(report[j]!=':'&&j<PI_REPORT_LENGTH){j++;}
	*pos_err=(atol(report+j+1)*PI_MPC[axis]);
	}

